unit fEditTools;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TfmEditTools = class(TForm)
    lbTools: TListBox;
    edCaption: TEdit;
    label1: TLabel;
    bSave: TButton;
    Label3: TLabel;
    edProg: TEdit;
    bOpen: TButton;
    bRemove: TButton;
    Bevel1: TBevel;
    bOK: TButton;
    Label2: TLabel;
    edParams: TEdit;
    Label4: TLabel;
    cbCaptureOutput: TCheckBox;
    label5: TLabel;
    edStartInDir: TEdit;
    lblParserRule: TLabel;
    edParserRule: TEdit;
    cbScrollToLastLine: TCheckBox;
    cbPause: TCheckBox;
    labExecOptParameters: TLabel;
    Label16: TLabel;
    labExecParam_n: TLabel;
    Label17: TLabel;
    labExecParam_f: TLabel;
    Label18: TLabel;
    labExecParam_ff: TLabel;
    Label31: TLabel;
    labExecParam_p: TLabel;
    Label6: TLabel;
    labExecParam_e: TLabel;
    Label9: TLabel;
    labExecParam_l: TLabel;
    Label10: TLabel;
    labExecParam_c: TLabel;
    Label11: TLabel;
    labExecParam_w: TLabel;
    Label24: TLabel;
    labExecParam_percent: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    procedure FormShow(Sender: TObject);
    procedure bSaveClick(Sender: TObject);
    procedure lbToolsClick(Sender: TObject);
    procedure bOpenClick(Sender: TObject);
    procedure bRemoveClick(Sender: TObject);
    procedure bOKClick(Sender: TObject);
    procedure cbCaptureOutputClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    OldCaption: string;
    procedure ReadIniFile;
    procedure ReadIniSection(section: string);
  public
    { Public declarations }
    lbtoolsidx: integer;
    function FindIdxByName(name: string): integer;
  end;


procedure fmEditToolsTryCreate;
function IniPath: string;

var
  fmEditTools: TfmEditTools;

implementation

{$R *.DFM}

uses unit1, uAppData, inifiles, AgOpenDialog, stringutils;

procedure fmEditToolsTryCreate;
begin
  if not assigned(fmEditTools) then
    fmEditTools := TfmEditTools.create(form1);
end;

function IniPath: string;
begin
  result := IncludeTrailingBackslash(GetAppdataFolder) + 'EditPlug\';
  CreateDir(result);
  result := result + 'plugins\';
  CreateDir(result);
  result := result + 'ExecTools\';
  CreateDir(result);
  result := result + 'tools.ini';
end;

procedure TfmEditTools.ReadIniFile;
var
  ini: TIniFile;
  sl: TStringList;
begin
  ini := TIniFile.create(IniPath);
  sl := TStringList.create;
  try
    ini.ReadSections(sl);
    sl.Sort;
    lbTools.Clear;
    lbTools.items.Assign(sl);
  finally
    ini.free;
    sl.free;
  end;
end;

procedure TfmEditTools.ReadIniSection(section: string);
var
  ini: TIniFile;
  readstr: string;
begin
  ini := TIniFile.create(IniPath);
  try
    edCaption.text := section;
    readstr := ini.ReadString(section, 'program', '');
    edProg.text := readstr;
    readstr := ini.ReadString(section, 'params', '');
    edParams.text := readstr;
    readstr := ini.ReadString(section, 'captureoutput', 'false');
    cbCaptureOutput.checked := strtobool(readstr);
    readstr := ini.ReadString(section, 'startindir', '');
    edStartInDir.text := readstr;
    readstr := ini.ReadString(section, 'parserule', '');
    edParserRule.text := readstr;
    readstr := ini.ReadString(section, 'scrolltolastline', 'false');
    cbScrollToLastLine.checked := strtobool(readstr);
    readstr := ini.ReadString(section, 'pause', 'false');
    cbPause.checked := strtobool(readstr);
  finally
    ini.free;
  end;
end;

procedure TfmEditTools.FormShow(Sender: TObject);
begin
  ReadIniFile;
  if lbTools.Items.Count > 0 then begin
    if lbTools.ItemIndex < 0 then lbTools.ItemIndex := lbtoolsidx;
    lbToolsClick(self);
  end;
end;

procedure TfmEditTools.bSaveClick(Sender: TObject);
var
  ini: TIniFile;
  idx: integer;

  function CaptionFound(s: string): boolean;
  var i: integer;
  begin
    result := false;
    for i := 0 to lbTools.Items.count -1 do begin
      // skip past own caption already displayed
      if i = lbTools.ItemIndex then continue;
      if lbTools.Items[i] = s then begin
        result := true;
        break;
      end;
    end;
  end;

var
  updated: boolean;
begin
  updated := false;
  idx := lbTools.ItemIndex;
  if idx < 0 then exit;
  ini := TIniFile.Create(inipath);
  try
    if CaptionFound(edCaption.text) then begin
      showmessage('The caption "'+ edCaption.text +'" is already in use by another tool');
    end else begin
      if OldCaption <> edCaption.text then ini.EraseSection(OldCaption);
      ini.WriteString(edCaption.text, 'program', edProg.text);
      ini.WriteString(edCaption.text, 'params', edParams.text);
      ini.WriteString(edCaption.text, 'captureoutput', booltostr(cbCaptureOutput.checked));
      ini.WriteString(edCaption.text, 'startindir', edStartInDir.text);
      ini.WriteString(edCaption.text, 'parserule', edParserRule.text);
      ini.WriteString(edCaption.text, 'scrolltolastline', booltostr(cbScrollToLastline.checked));
      ini.WriteString(edCaption.text, 'pause', booltostr(cbPause.checked));            
      updated := true;
    end;
  finally
    ini.free;
    if updated then begin
      ReadIniFile;
      lbTools.ItemIndex := FindIdxByName(edCaption.text);
    end;
  end;
end;

procedure TfmEditTools.lbToolsClick(Sender: TObject);
var idx: integer;
begin
  idx := lbTools.ItemIndex;
  if idx < 0 then exit;
  ReadIniSection(lbTools.items[idx]);
  OldCaption := edCaption.text
end;

procedure TfmEditTools.bOpenClick(Sender: TObject);
var dlgOpen:TAgOpenDialog;
begin
  dlgOpen:=TAgOpenDialog.Create(SELF);
  try
    dlgOpen.DefaultExt:='exe';
    dlgOpen.Filter:='Executable files|*.exe;*.com;*.bat|All files (*.*)|*.*';
    if (dlgOpen.Execute) then edProg.Text:=dlgOpen.FileName;
  finally
    dlgOpen.Free;
    self.show;
  end;
end;

procedure TfmEditTools.bRemoveClick(Sender: TObject);
var
  ini: TIniFile;
begin
  ini := TIniFile.create(IniPath);
  try
    ini.EraseSection(edCaption.text);
  finally
    ini.free;
  end;
  ReadIniFile;
  if lbTools.Items.count > 0 then lbTools.ItemIndex := 0;
  lbToolsClick(self);
end;

function TfmEditTools.FindIdxByName(name: string): integer;
var i: integer;
begin
  result := -1;
  for i := 0 to lbTools.items.count -1 do begin
    if lbTools.items[i] = name then begin
      result := i;
      break;
    end;
  end;
end;

procedure TfmEditTools.bOKClick(Sender: TObject);
begin
  bSaveClick(self);
  form1.MakeButtonsFromIni;
  ModalResult := 1;
  Self.close;
end;

procedure TfmEditTools.cbCaptureOutputClick(Sender: TObject);
begin
  lblParserRule.enabled := cbCaptureOutput.checked;
  edParserRule.enabled := cbCaptureOutput.checked;
  cbPause.enabled := not cbCaptureOutput.checked;
  cbScrollToLastLine.Enabled := cbCaptureOutput.checked;
end;

procedure TfmEditTools.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  lbtoolsidx := 0;
end;

end.
